#include "StdAfx.h"
#include ".\myimage.h"

CMyImage::CMyImage(void)
: m_strFileName(_T(""))
{
	m_pPicture = NULL;
}

CMyImage::~CMyImage(void)
{
}

HRESULT CMyImage::FileToStream(LPCTSTR szFile, IStream** ppStream)
{
	HRESULT hr;
	HGLOBAL hGlobal = NULL;
	// open file
	HANDLE hFile = ::CreateFile(szFile, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (INVALID_HANDLE_VALUE == hFile)
		return HRESULT_FROM_WIN32(::GetLastError());

	// get file size
	DWORD dwBytesRead = 0, dwFileSize = ::GetFileSize(hFile, NULL);
	if (-1 != dwFileSize)
	{
		hGlobal = ::GlobalAlloc(GMEM_MOVEABLE, dwFileSize);
		if (NULL != hGlobal)
		{
			PVOID pvData = ::GlobalLock(hGlobal);
			ATLASSERT(NULL != pvData);

			// read file and store in global memory
			::ReadFile(hFile, pvData, dwFileSize, &dwBytesRead, NULL);
			::GlobalUnlock(hGlobal);

			if (dwFileSize != dwBytesRead)
			{
				::GlobalFree(hGlobal);
				hr = E_FAIL;
			}
			else
				hr = S_OK;
		}
		else
			hr = E_OUTOFMEMORY;
	}
	else
		hr = HRESULT_FROM_WIN32(::GetLastError());

	::CloseHandle(hFile);

	// create IStream* from global memory
	if (SUCCEEDED(hr))
		hr = ::CreateStreamOnHGlobal(hGlobal, TRUE, ppStream);
	return hr;
}

HRESULT CMyImage::DrawImage(HDC hdc, const RECT& rcBounds)
{
	if(m_pPicture != NULL)
	{
			// Get the width and the height of the picture
			long hmWidth = 0, hmHeight = 0;
			m_pPicture->get_Width(&hmWidth);
			m_pPicture->get_Height(&hmHeight);

			// Convert himetric to pixels
			//int nWidth	= MulDiv(hmWidth, ::GetDeviceCaps(hdc, LOGPIXELSX), 0/*HIMETRIC_INCH*/);
			//int nHeight	= MulDiv(hmHeight, ::GetDeviceCaps(hdc, LOGPIXELSY), 0/*HIMETRIC_INCH*/);

			// Display the picture using IPicture::Render
			return m_pPicture->Render(
				hdc,							// [in] Handle of device context on which to render the image
				rcBounds.left,					// [in] Horizontal position of image in hdc
				rcBounds.top,					// [in] Vertical position of image in hdc
				rcBounds.right - rcBounds.left,	// [in] Horizontal dimension of destination rectangle
				rcBounds.bottom - rcBounds.top, // [in] Vertical dimension of destination rectangle
				0,								// [in] Horizontal offset in source picture
				hmHeight,						// [in] Vertical offset in source picture
				hmWidth,						// [in] Amount to copy horizontally in source picture
				-hmHeight,						// [in] Amount to copy vertically in source picture
				&rcBounds						// [in, optional] Pointer to position of destination for a metafile hdc
				);
	}

		return E_UNEXPECTED;
}


HRESULT CMyImage::LoadFileImage(LPCTSTR szFileName)
{
	CComPtr<IStream> pStream;
	m_strFileName = szFileName;	

	// Load the file to a memory stream
	HRESULT hr = FileToStream(szFileName, &pStream);
	if(m_pPicture != NULL)
		m_pPicture = NULL;
	if (SUCCEEDED(hr))
	{
		// Decode the picture
		hr = ::OleLoadPicture(
					pStream,			// [in] Pointer to the stream that contains picture's data
					0,					// [in] Number of bytes read from the stream (0 == entire)
					true,				// [in] Loose original format if true
					IID_IPicture,		// [in] Requested interface
					(void**)&m_pPicture // [out] IPictire object on success
					);
	}

	return hr;
}

HRESULT CMyImage::DeleteImage(void)
{
	if(m_pPicture != NULL)
		m_pPicture = NULL;

	return E_NOTIMPL;
}

bool CMyImage::GetImageFileName(CString& strFileName)
{
	if(m_pPicture != NULL){
		strFileName = m_strFileName;
		return true;
	}
	return false;
}
